import Color from 'color';
import { lowerLuminosity, raiseLuminosity } from './luminosity';
import ms from 'modularscale';

/*
 * Based on the blog post:
 *
 *   https://medium.com/@erikdkennedy/color-in-ui-design-a-practical-framework-e18cacd97f9e
 *
 * The article does not prescribe an exact algorithum for adjusting the
 * individual color values, so I reached for the modularscale module as a
 * starting point. Changing the "step" introduces more variation between
 * adjustments and choosing a different ratio scale can increase or decrease
 * the amounts between those steps.
 */

export function darken(baseColor) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$alpha = _ref.alpha,
      alpha = _ref$alpha === undefined ? null : _ref$alpha,
      _ref$format = _ref.format,
      format = _ref$format === undefined ? 'rgb' : _ref$format,
      scale = _ref.scale,
      _ref$steps = _ref.steps,
      steps = _ref$steps === undefined ? 0 : _ref$steps;

  if (!baseColor) {
    return baseColor;
  }
  var c = Color(baseColor);
  var delta = ms(steps, scale) / 100;
  var newSaturation = c.saturationv() + c.saturationv() * delta;
  var newValue = c.value() - c.value() * delta;
  alpha = alpha || c.alpha();

  return c.hue(lowerLuminosity(c.hue(), delta)).saturationv(newSaturation).value(newValue).alpha(alpha)[format]().string();
}

export function lighten(baseColor) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref2$alpha = _ref2.alpha,
      alpha = _ref2$alpha === undefined ? null : _ref2$alpha,
      _ref2$format = _ref2.format,
      format = _ref2$format === undefined ? 'rgb' : _ref2$format,
      scale = _ref2.scale,
      _ref2$steps = _ref2.steps,
      steps = _ref2$steps === undefined ? 0 : _ref2$steps;

  if (!baseColor) {
    return baseColor;
  }
  var c = Color(baseColor);
  var delta = ms(steps, scale) / 100;
  var newSaturation = c.saturationv() - c.saturationv() * delta;
  var newValue = c.value() + c.value() * delta;
  alpha = alpha || c.alpha();

  return c.hue(raiseLuminosity(c.hue(), delta)).saturationv(newSaturation).value(newValue).alpha(alpha)[format]().string();
}